<?php

abstract class WPSC_Paybox_Abstract_Gateway {

    protected $_config;
    protected $_paybox;
    protected $_orderClass = 'WPSC_Purchase_Log';
    public $settings_id = 'paybox_';

    public function __construct() {
        $this->_config = new Paybox_Config(array($this->id), $this->defaultTitle, $this->defaultDesc);
        // Load the settings
        $this->init_settings();
        $this->_config = new Paybox_Config($this->settings, $this->defaultTitle, $this->defaultDesc);
        $this->_paybox = new Payboxclass($this->_config);
        $this->title = $this->_config->getTitle();
        $this->description = $this->_config->getDescription();
    }

    public function format_settings($value) {
        return is_array($value) ? $value : $value;
    }

    public function init_settings() {

        // Load form_field settings
        $this->settings = get_option($this->settings_id . $this->type . '_settings', null);

        if (!$this->settings || !is_array($this->settings)) {

            $this->settings = array();

            // If there are no settings defined, load defaults
            if ($form_fields = $this->_getFormFields()) {

                foreach ($form_fields as $k => $v) {
                    $this->settings[$k] = isset($v['default']) ? $v['default'] : '';
                }
            }
        }

        if (!empty($this->settings) && is_array($this->settings)) {
            $this->settings = array_map(array($this, 'format_settings'), $this->settings);
        }
    }

    private function _getFormFields() {
        return $this->_config->init_form_fields($this->type);
    }

    public function getGatewayInformation() {
        return array(
            'name' => __($this->defaultTitle, 'paybox'),
            'internalname' => $this->id,
            'function' => 'nzshpcrt_gateway_' . $this->id,
            'form' => 'nzshpcrt_form_' . $this->id,
            'submit_function' => 'nzshpcrt_submit_' . $this->id,
            'payment_type' => 'credit_card',
            'display_name' => $this->_config->getTitle(),
            'class_name' => get_class($this),
            'has_recurring_billing' => true,
            'wp_admin_cannot_cancel' => true,
        );
    }

    public function admin_form() {
        $form_fields = $this->_getFormFields();
        $g = new WPSC_Paybox_Admin_Form();
        $crypt = new PayboxEncrypt();
        if (isset($this->settings['hmackey'])) {
            $this->settings['hmackey'] = $crypt->decrypt($this->settings['hmackey']);
        }
        $output = $g->render($form_fields, $this->id, $this->settings);
        $output .= '<script type="text/javascript">
(function($, gateway) {
	$(document).ready(function() {
		function on3dsEnabledChange() {
			var amountRow = $($("[name=" + gateway + "_3ds_amount]").parents("tr")[0]);
			if ($(this).val() == "conditional") {
				amountRow.show();
			}
			else {
				amountRow.hide();
			}
		}
		enabled = $("[name=" + gateway + "_3ds_enabled]");
		enabled.change(on3dsEnabledChange);
		setTimeout(function() {
			on3dsEnabledChange.apply(enabled);
		}, 10);
	});
})(jQuery, "' . esc_js($this->id) . '");
</script>';
        //$($("[name=" + gateway + "_debug]").parents("tr")[0]).hide();
        //$($("[name=" + gateway + "_ips]").parents("tr")[0]).hide();

        return $output;
    }

    public function admin_save() {
        $form_fields = $this->_getFormFields();
        foreach ($form_fields as $name => $def) {
            if ($def['type'] != 'title') {
                $realName = $this->id . '_' . $name;
                if (isset($_POST[$realName])) {
                    update_option($realName, stripslashes($_POST[$realName]));
                } else if ($def['type'] == 'checkbox') {
                    update_option($realName, 'no');
                }
            }
        }
        return true;
    }

    /**
     * Process the payment, redirecting user to Paybox.
     */
    public function process_payment($separator, $sessionId) {
        $order = new WPSC_Purchase_Log($sessionId, 'sessionid');

        $message = $this->_paybox->_messages['info']['paybox-customer-redirect'];
        $this->_paybox->addOrderNote($order, $message);

        $baseUrl = add_query_arg('paybox_callback', $this->id, get_site_url());
        $urls = array(
            'PBX_ANNULE' => add_query_arg('s', 'c', $baseUrl),
            'PBX_EFFECTUE' => add_query_arg('s', 's', $baseUrl),
            'PBX_REFUSE' => add_query_arg('s', 'f', $baseUrl),
            'PBX_REPONDRE_A' => add_query_arg('s', 'i', $baseUrl),
        );

        $params = $this->_paybox->buildSystemParams($order, $this->type, $urls);

        $url = $this->_paybox->getSystemUrl();
        $debug = $this->_config->isDebug();

        $debugViewMsg = $this->_paybox->_messages['info']['paybox-debug-view'];
        $redirectButtonMsg = $this->_paybox->_messages['info']['paybox-redirect-button'];
        $continueMsg = $this->_paybox->_messages['info']['paybox-continue'];
        ?>
        <!doctype html>
        <html>
            <head>
                <meta charset="utf-8"/>
            </head>
            <body>
                <form id="pbxep_form" method="post" action="<?php echo esc_url($url); ?>" enctype="application/x-www-form-urlencoded">
                    <?php if ($debug): ?>
                        <p><center>
                            <?php echo $debugViewMsg; ?>
                        </center></p>
                <?php else: ?>
                    <p><center>
                        <?php echo $redirectButtonMsg; ?>
                    </center></p>
            <?php endif; ?>
            <center><button><?php echo $continueMsg; ?></button></center>
            <?php
            $type = $debug ? 'text' : 'hidden';
            foreach ($params as $name => $value):
                $name = esc_attr($name);
                $value = esc_attr($value);
                if ($debug):
                    echo '<p><label for="' . $name . '">' . $name . '</label>';
                endif;
                echo '<input type="' . $type . '" id="' . $name . '" name="' . $name . '" value="' . $value . '" />';
                if ($debug):
                    echo '</p>';
                endif;
            endforeach;
            ?>
        </form>
        <?php if (!$debug): ?>
            <script type="text/javascript">
                window.setTimeout(function () {
                    document.getElementById('pbxep_form').submit();
                }, 1);
            </script>
        <?php endif; ?>
        </body>
        </html>
        <?php
        exit();
    }

    public function api_call() {
        if (!isset($_GET['s'])) {
            header('Status: 404 Not found', true, 404);
            die('Not found');
        }

        try {
            switch ($_GET['s']) {
                case 'c':
                    return $this->_paybox->on_payment_canceled($this->_orderClass);
                    break;

                case 'f':
                    return $this->_paybox->on_payment_failed($this->_orderClass);
                    break;

                case 'i':
                    return $this->_paybox->on_ipn($this->_orderClass, $this->type);
                    break;

                case 's':
                    return $this->_paybox->on_payment_succeed($this->_orderClass);
                    break;

                default:
                    header('Status: 404 Not found', true, 404);
                    die('Not found');
            }
        } catch (Exception $e) {
            header('Status: 500 Error', true, 500);
            die($e->getMessage());
        }
    }

    public function showDetails($orderId) {
        $this->_paybox->showDetails($orderId, $this->_orderClass, $this->type);
    }

}
