<?php

/**
 * Plugin Name: WP eCommerce Paybox Payment gateway
 * Description: WP eCommerce gateway payment plugin for Paybox
 * Version: 1.0.0.0
 * Author: Paybox Verifone
 * Author URI: http://www.paybox.com
 * 
 * @package WordPress
 */
// Ensure not called directly
if (!defined('ABSPATH')) {
    exit;
}

include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
include_once( ABSPATH . 'wp-admin/includes/template.php' );

define('WPSC_PAYBOX_PLUGIN', 'wpsc_paybox');
define('WPSC_PAYBOX_VERSION', '1.0.0.0');
if (!defined('PAYBOX_INCLUDES_PATH'))
    define('PAYBOX_INCLUDES_PATH', WP_PLUGIN_DIR . '/paybox/includes/');

if (!defined('PAYBOX_KEY_PATH'))
    define('PAYBOX_KEY_PATH', ABSPATH . 'kek.php');

function check_paybox_state_wpe() {
    $this_plugin = get_plugin_data(__FILE__);
    $plugins = array(
        'paybox' => array(
            'file_path' => 'paybox/paybox.php',
            'required' => true,
            'name' => 'Wordpress Paybox Payment plugin',
            'slug' => 'paybox',
            'author' => 'Paybox Verifone',
            'depend' => '',
            'external_url' => 'http://www1.paybox.com/espace-integrateur-documentation/modules-by-paybox/',
            'wordpress_org_name' => 'paybox',
            'source' => false,
        ),
    );
    $installed_plugins = get_plugins();

    if (!isset($installed_plugins[$plugins['paybox']['file_path']])) {
        $external_url = $plugins['paybox']['external_url'];
        $source = $plugins['paybox']['source'];
        $link = '';
        if ($external_url && preg_match('|^http(s)?://|', $external_url)) {
            $link = '<a href="' . esc_url($external_url) . '" title="' . $plugins['paybox']['name'] . '" target="_blank">' . $plugins['paybox']['name'] . '</a>';
        } elseif (!$source || preg_match('|^http://wordpress.org/extend/plugins/|', $source)) {
            $url = add_query_arg(
                    array(
                'tab' => 'plugin-information',
                'plugin' => $plugins['paybox']['wordpress_org_name'],
                'TB_iframe' => 'true',
                'width' => '640',
                'height' => '500',
                    ), network_admin_url('plugin-install.php')
            );

            $link = '<a href="' . esc_url($url) . '" class="thickbox" title="' . $plugins['paybox']['name'] . '">' . $plugins['paybox']['name'] . '</a>';
        } else {
            $link = $plugins['paybox']['name']; // No hyperlink.
        }

        $message = '<i>' . $this_plugin['Name'] . '</i>' . ' requiert le plugin suivant : ' . $link;
        add_settings_error('paybox-wpe', 'paybox-wpe', $message, 'error');
        settings_errors('paybox-wpe');
        return false;
    }

    if (is_plugin_inactive($plugins['paybox']['file_path'])) {
        require_once(PAYBOX_INCLUDES_PATH . 'paybox-helper.php');
        $plugins = Paybox_Helper::getPluginGeneric();
        Paybox_Helper::notices($plugins);
        return false;
    }

    return true;
}

function wpsc_paybox_installation() {
    $currentVersion = get_option(WPSC_PAYBOX_PLUGIN . '_version');
    if (($currentVersion === false) || (version_compare($currentVersion, WPSC_PAYBOX_VERSION) < 0)) {
        global $wpdb;
        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        $tableName = $wpdb->prefix . 'wpsc_paybox_message';
        $sql = "CREATE TABLE $tableName ( 
		id INT(10) NOT NULL AUTO_INCREMENT, 
		order_id BIGINT(20) NOT NULL, 
		date DATETIME NOT NULL, 
		message VARCHAR(512) NOT NULL, 
		PRIMARY KEY  (id), 
		KEY order_id (order_id) 
		);";
        dbDelta($sql);

        $tableName = $wpdb->prefix . 'wpsc_paybox_payment';
        $sql = "CREATE TABLE $tableName ( 
		id INT(10) NOT NULL AUTO_INCREMENT, 
		order_id BIGINT(20) NOT NULL, 
		type ENUM('capture','first_payment','second_payment','third_payment') NOT NULL, 
		data VARCHAR(2048) NULL DEFAULT NULL, 
		PRIMARY KEY  (id), 
		KEY order_id (order_id) 
		)";
        dbDelta($sql);

        update_option(WPSC_PAYBOX_PLUGIN . '_version', WPSC_PAYBOX_VERSION);
    }
}

function wpsc_paybox_initialization() {
    require_once(PAYBOX_INCLUDES_PATH . 'paybox-config.php');
    require_once(PAYBOX_INCLUDES_PATH . 'paybox-iso4217currency.php');
    require_once(PAYBOX_INCLUDES_PATH . 'payboxclass.php');
    require_once(plugin_dir_path(__FILE__) . '/includes/wpsc-paybox-abstract-gateway.php');
    require_once(plugin_dir_path(__FILE__) . '/includes/wpsc-paybox-standard-gateway.php');
    require_once(plugin_dir_path(__FILE__) . '/includes/wpsc-paybox-threetime-gateway.php');
    require_once(plugin_dir_path(__FILE__) . '/includes/wpsc-paybox-admin-form.php');

    load_plugin_textdomain('paybox', false, 'paybox/lang');

    // Create payment methods
    global $nzshpcrt_gateways_paybox;
    $nzshpcrt_gateways_paybox = new WPSC_Paybox_Standard_Gateway();
    global $nzshpcrt_gateways_paybox3x;
    $nzshpcrt_gateways_paybox3x = new WPSC_Paybox_Threetime_Gateway();

// Register payment methods
    global $nzshpcrt_gateways, $wpsc_gateways;
    $num = 99;
    $nzshpcrt_gateways[$num] = $nzshpcrt_gateways_paybox->getGatewayInformation();
    $nzshpcrt_gateways[++$num] = $nzshpcrt_gateways_paybox3x->getGatewayInformation();
    $num = 99;
    $wpsc_gateways[$nzshpcrt_gateways_paybox->getGatewayInformation()['internalname']] = &$nzshpcrt_gateways[$num];
    $wpsc_gateways[$nzshpcrt_gateways_paybox3x->getGatewayInformation()['internalname']] = &$nzshpcrt_gateways[++$num];
}

// Wrapper hooks
function nzshpcrt_gateway_paybox_std($separator, $sessionId) {
    global $nzshpcrt_gateways_paybox;
    return $nzshpcrt_gateways_paybox->process_payment($separator, $sessionId);
}

function nzshpcrt_form_paybox_std() {
    global $nzshpcrt_gateways_paybox;
    return $nzshpcrt_gateways_paybox->admin_form();
}

function nzshpcrt_submit_paybox_std() {
    global $nzshpcrt_gateways_paybox;
    return $nzshpcrt_gateways_paybox->admin_save();
}

function nzshpcrt_gateway_paybox_3x($separator, $sessionId) {
    global $nzshpcrt_gateways_paybox3x;
    return $nzshpcrt_gateways_paybox3x->process_payment($separator, $sessionId);
}

function nzshpcrt_form_paybox_3x() {
    global $nzshpcrt_gateways_paybox3x;
    return $nzshpcrt_gateways_paybox3x->admin_form();
}

function nzshpcrt_submit_paybox_3x() {
    global $nzshpcrt_gateways_paybox3x;
    return $nzshpcrt_gateways_paybox3x->admin_save();
}

function nzshpcrt_paybox_callback() {
    global $nzshpcrt_gateways_paybox;
    global $nzshpcrt_gateways_paybox3x;
    if (isset($_GET['paybox_callback']) && isset($_GET['s']) && in_array($_GET['paybox_callback'], array('paybox_std', 'paybox_3x'))) {
        if ($_GET['paybox_callback'] == 'paybox_std') {
            $nzshpcrt_gateways_paybox->api_call();
        } else {
            $nzshpcrt_gateways_paybox3x->api_call();
        }
    }
}

function nzshpcrt_paybox_is_available(array $gateways) {
    global $wpsc_cart;
    global $nzshpcrt_gateways_paybox;
    global $nzshpcrt_gateways_paybox3x;
    $result = array();
    for ($i = 0; $i < count($gateways); ++$i) {
        if (isset($gateways[$i]['internalname'])) {
            if ($gateways[$i]['internalname'] == 'paybox_3x') {
                if (!$nzshpcrt_gateways_paybox3x->isAvailable()) {
                    continue;
                }
            }
        }
        $result[] = $gateways[$i];
    }
    return $result;
}

function nzshpcrt_paybox_show_details() {
    global $purchlogitem;
    global $nzshpcrt_gateways_paybox;
    global $nzshpcrt_gateways_paybox3x;
    switch ($purchlogitem->extrainfo->gateway) {
        case 'paybox_std':
            $nzshpcrt_gateways_paybox->showDetails($purchlogitem->extrainfo->id);
            break;
        case 'paybox_3x':
            $nzshpcrt_gateways_paybox3x->showDetails($purchlogitem->extrainfo->id);
            break;
    }
}

if (check_paybox_state_wpe()) {
    register_activation_hook(__FILE__, 'wpsc_paybox_installation');
    add_action('plugins_loaded', 'wpsc_paybox_initialization');

    add_action('init', 'nzshpcrt_paybox_callback');
    add_filter('wpsc_merchant_v2_gateway_loop_items', 'nzshpcrt_paybox_is_available');
    add_filter('wpsc_billing_details_bottom', 'nzshpcrt_paybox_show_details', 10, 3);
}
